'use strict';

var fs = require('fs');
var upath = require('upath');
var lodash = require('lodash');
var loaderUtils = require('loader-utils');

function parseSfc$1(compiler) {
  var filename = upath.basename(this.resourcePath);
  var context = this.rootContext || process.cwd();
  var sourceRoot = upath.dirname(upath.relative(context, this.resourcePath));
  return compiler.parse(fs.readFileSync("".concat(sourceRoot, "/").concat(filename), 'utf8')).descriptor;
}

function compileTemplateFromDescriptor$1(sfcDescriptor, compiler) {
  var _sfcDescriptor$templa;

  // check we have a template to work with
  if (!(sfcDescriptor !== null && sfcDescriptor !== void 0 && (_sfcDescriptor$templa = sfcDescriptor.template) !== null && _sfcDescriptor$templa !== void 0 && _sfcDescriptor$templa.content)) return;
  var content = sfcDescriptor.template.content; // need to compile pug to html so that we can compile it using the vue/compiler-sfc

  if (sfcDescriptor.template.lang === 'pug') {
    var pug = require('pug');

    content = pug.render(content, {
      filename: this.resourcePath
    });
  }

  var shortFilePath = this.resourcePath.replace(/^(\.\.[\/\\])+/, '').replace(/\\/g, '/');
  return compiler.compileTemplate({
    id: shortFilePath,
    source: content,
    filename: this.resourcePath,
    prettify: false
  });
}

function extractTagsFromSfc$1(options) {
  // parse the SFC component and get a descriptor
  var sfcDescriptor = parseSfc$1.call(this, options.compiler); // compile the template content from the descriptor

  var compiled = compileTemplateFromDescriptor$1.call(this, sfcDescriptor, options.compiler);

  if (compiled && compiled.ast) {
    var tags = compiled.ast.components; // return all unique tags as kebab case

    return lodash.uniq(lodash.map(tags, function (tag) {
      return lodash.kebabCase(tag);
    }));
  }

  return false;
}

function injectImports(variable, components) {
  return "".concat(variable, ".components = Object.assign({}, { ").concat(components.map(function (c) {
    return c.lazy ? c["import"] : c.pascalName;
  }).join(', '), " }, ").concat(variable, ".components);");
}

function injectComponents$1(source, components) {
  var lazyCode = !source.includes('defineAsyncComponent') && components.filter(function (c) {
    return c.lazy;
  }).length > 0 ? 'import { defineAsyncComponent } from \'vue\'' : '';
  var newContent = "/* vue-cli-plugin-import-components */\n".concat(lazyCode, "\n").concat(components.filter(function (c) {
    return !c.lazy;
  }).map(function (c) {
    return c["import"];
  }).join('\n'), "\n").concat(injectImports('script', components)); // script.options is used by vue-property-decorator

  newContent += "if (script.options) { ".concat(injectImports('script.options', components), "}");
  var hotReload = source.indexOf('/* hot reload */');
  if (hotReload > -1) source = "".concat(source.slice(0, hotReload) + newContent, "\n\n").concat(source.slice(hotReload));else source += "\n\n".concat(newContent);
  return source;
}

function parseSfc(compiler) {
  var filename = upath.basename(this.resourcePath);
  var context = this.rootContext || process.cwd();
  var sourceRoot = upath.dirname(upath.relative(context, this.resourcePath));
  return compiler.parseComponent(fs.readFileSync("".concat(sourceRoot, "/").concat(filename), 'utf8'));
}

function compileTemplateFromDescriptor(sfcDescriptor, compiler) {
  var _sfcDescriptor$templa;

  // check we have a template to work with
  if (!(sfcDescriptor !== null && sfcDescriptor !== void 0 && (_sfcDescriptor$templa = sfcDescriptor.template) !== null && _sfcDescriptor$templa !== void 0 && _sfcDescriptor$templa.content)) return;
  var content = sfcDescriptor.template.content; // need to compile pug to html so that we can compile it using the vue/compiler-sfc

  if (sfcDescriptor.template.lang === 'pug') {
    var pug = require('pug');

    content = pug.render(content, {
      filename: this.resourcePath
    });
  }

  var tags = [];
  var compiled = compiler.compile(content, {
    modules: [{
      postTransformNode: function postTransformNode(node) {
        tags.push(node.tag);
      }
    }]
  });
  compiled.tags = tags;
  return compiled;
}

function extractTagsFromSfc(options) {
  // parse the SFC component and get a descriptor
  var sfcDescriptor = parseSfc.call(this, options.compiler); // compile the template content from the descriptor

  var compiled = compileTemplateFromDescriptor.call(this, sfcDescriptor, options.compiler);

  if (compiled && compiled.tags) {
    // return all unique tags as kebab case
    return lodash.uniq(lodash.map(compiled.tags, function (tag) {
      return lodash.kebabCase(tag);
    }));
  }

  return false;
}

var installComponents = require.resolve('./runtime/installComponents');

function injectComponents(source, components) {
  var installComponentRuntime = "const vueCliInstallComponent = require(".concat(loaderUtils.stringifyRequest(this, "!".concat(installComponents)), ")");
  var injectScript = "/* vue-cli-plugin-import-components */\n".concat(installComponentRuntime, "\n").concat(components.filter(function (c) {
    return !c.lazy;
  }).map(function (c) {
    return c["import"];
  }).join('\n'), "\nvueCliInstallComponent(component, {").concat(components.map(function (c) {
    return c.lazy ? c["import"] : c.pascalName;
  }).join(', '), "});");
  var hotReload = source.indexOf('/* hot reload */');
  if (hotReload > -1) source = "".concat(source.slice(0, hotReload) + injectScript, "\n\n").concat(source.slice(hotReload));else source += "\n\n".concat(injectScript);
  return source;
}

var _require = require('@vue/cli-shared-utils'),
    semver = _require.semver,
    loadModule = _require.loadModule,
    error = _require.error,
    info = _require.info;

function loadVue2TemplateCompiler(api) {
  return loadModule('vue-template-compiler', api.service.context) || loadModule('vue-template-compiler', __dirname);
}

function loadVue3Compiler(api) {
  return loadModule('@vue/compiler-sfc', api.service.context) || loadModule('@vue/compiler-sfc', __dirname);
}

var plugin = function plugin(api, options) {
  var _options$pluginOption;

  var extensions = ['vue', 'js', 'ts'];
  var pluginOptions = Object.assign( // default configuration
  {
    path: './src/components',
    extensions: extensions,
    ignore: ['**/*.stories.js']
  }, // users provided configuration
  (_options$pluginOption = options.pluginOptions) === null || _options$pluginOption === void 0 ? void 0 : _options$pluginOption.components);
  if (!pluginOptions.pattern) pluginOptions.pattern = "**/*.{".concat(pluginOptions.extensions.join(','), ",}"); // resolve the configured path

  pluginOptions.path = api.resolve(pluginOptions.path); // check the components path exists

  if (!fs.existsSync(pluginOptions.path)) {
    // if not we should create it so we don't error out
    fs.mkdirSync(pluginOptions.path); // warn the user, possible misconfiguration?

    info("The components path \"".concat(pluginOptions.path, "\" was created."), 'vue-cli-plugin-import-components');
  }

  var vue = loadModule('vue', api.service.context) || loadModule('vue', __dirname);

  if (!vue) {
    error('Aborting, failed to load vue module.', 'vue-cli-plugin-import-components');
    return;
  }

  var vueVersion = semver.major(vue.version);
  pluginOptions.vueVersion = vueVersion;

  if (vueVersion === 2) {
    pluginOptions.extractor = extractTagsFromSfc;
    pluginOptions.injector = injectComponents;
    pluginOptions.compiler = loadVue2TemplateCompiler(api);
  } else if (vueVersion === 3) {
    pluginOptions.extractor = extractTagsFromSfc$1;
    pluginOptions.injector = injectComponents$1;
    pluginOptions.compiler = loadVue3Compiler(api);
  } else {
    error("Aborting, vue version ".concat(vueVersion, " not supported"), 'vue-cli-plugin-import-components');
    return;
  } // extend the base webpack configuration


  api.chainWebpack(function (webpackConfig) {
    // @todo use oneOf so we're not running the loader when it's not required?
    webpackConfig.module.rules.get('vue') // add our custom components loader before the vue-loader
    .use('components').loader(require.resolve('./loader')).options(pluginOptions).before('vue-loader').end();
  }); // need to return something for typescript

  return true;
};

module.exports = plugin;
